from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from MainWindow import Ui_MainWindow

from OtherWindowFuncs import *
from Models import *
from pfc_func import *

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s


class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.setupUi(self)

        #Crea les finestres 
        self.aboutwin = About()
        self.globalsettingswin = GlobalSettings()
        self.toolpreferenceswin = ToolPreferences()
        self.advancedsearchwin = AdvancedSearch(self)

        #Variables per al control de les dades dels SpinBox retocats
        self.zoom = 0
        self.sort1 = 0
        self.sort2 = 0
        self.sort3 = 0
        
        #Contindrà les seleccions de paraules en el text del tab File View
        self.ex = []
        
        #Contindrà els resultats de les cerques en el tab Concordance
        self.results = []
        """results és un llistat, on cada element conté la informació següent sobre cada coincidència:
        posició de l'expressió a l'arxiu
        expressió
        nom de l'arxiu
        linia original complerta
        Posició de l'expressió a la linia        
        tupla amb posició inici i final sortL1 a la linia
        tupla amb posició inici i final sortL2 a la linia
        tupla amb posició inici i final sortL3 a la linia"""
        
        #Creació dels models
        self.filesModel = FileListModel(self)
        self.kFModel = kwikFModel(self)
        self.kKModel = kwikKModel(self)
        self.hModel = hitModel(self)
        self.WLrankModel = WLrankModel(self)
        self.WLfreqModel = WLfreqModel(self)
        self.WLwordModel = WLwordModel(self)
        self.WLlemmaModel = WLlemmaModel(self)
        
        #Assignació dels models als elements
        self.fileList.setModel(self.filesModel)
        self.concFileList.setModel(self.kFModel)
        self.concKwikList.setModel(self.kKModel)
        self.concHitList.setModel(self.hModel)
        self.WLrankList.setModel(self.WLrankModel)
        self.WLfreqList.setModel(self.WLfreqModel)
        self.WLwordList.setModel(self.WLwordModel)
        self.WLlemmaList.setModel(self.WLlemmaModel)
        
        #Les següents llistes acceptaran HTML tal com es defineix a HTMLDelegate (a Models.py)
        self.concKwikList.setItemDelegate(HTMLDelegate(self))
        self.concFileList.setItemDelegate(HTMLDelegate(self))
        self.concHitList.setItemDelegate(HTMLDelegate(self))
        
        #Definim l'ample inicial de les diferents columnes
        self.ConcordanceSplitter.setSizes([50,450,150])    #Estableix les mides dels panells al splitter de Concordance
        self.CollocatesSplitter.setSizes([60,60,60,60,500])    #Estableix les mides dels panells al splitter de Collocates
        self.WordListSplitter.setSizes([60,60,200,500])    #Estableix les mides dels panells al splitter de Collocates

        #Defineix com s'insertaran els items en searchTerm
        #1 vol dir que s'insertaran a l'inici de la llista i no al final
        self.searchTerm.setInsertPolicy(1)
 
        #Amaguem les vertical scrollBar a totes les llistes de concordance excepte la darrera.
        #Fem que totes les llistes facin scroll amb aquesta darrera scrollbar.
        self.concKwikList.setVerticalScrollBar(self.concHitList.verticalScrollBar())
        self.concFileList.setVerticalScrollBar(self.concKwikList.verticalScrollBar())
        #L'ordre és molt important. El darrer verticalScrollbar assignat serà el que mostrarà la barra.
        #Els altres 2 no mostren la barra. És a dir que la scrollbar queda a concFileList.
        
        #Fem el mateix amb les llistes de Word List
        self.WLfreqList.setVerticalScrollBar(self.WLrankList.verticalScrollBar())
        self.WLwordList.setVerticalScrollBar(self.WLfreqList.verticalScrollBar())
        self.WLlemmaList.setVerticalScrollBar(self.WLwordList.verticalScrollBar())
        
        
        #Creem els botons de la barra superior com Actions. 
        #Si es feien com menus en Qt Designer, en pasar per sobre d'un o altre s'obrien els diferents quadres
        
        #Botó Global Settings
        self.buttonGlobalSettings = QtGui.QAction(self.menubar)
        self.buttonGlobalSettings.setObjectName(_fromUtf8("buttonGlobalSettings"))
        self.menubar.addAction(self.buttonGlobalSettings)
        self.buttonGlobalSettings.setText(QtGui.QApplication.translate("MainWindow", "Global Settings", None, QtGui.QApplication.UnicodeUTF8))

        #Botó Tool Preferences
        self.buttonToolPreferences = QtGui.QAction(self.menubar)
        self.buttonToolPreferences.setObjectName(_fromUtf8("buttonToolPreferences"))
        self.menubar.addAction(self.buttonToolPreferences)
        self.buttonToolPreferences.setText(QtGui.QApplication.translate("MainWindow", "Tool Preferences", None, QtGui.QApplication.UnicodeUTF8))

        #Botó About
        self.buttonAbout = QtGui.QAction(self.menubar)
        self.buttonAbout.setObjectName(_fromUtf8("buttonAbout"))
        self.menubar.addAction(self.buttonAbout)
        self.buttonAbout.setText(QtGui.QApplication.translate("MainWindow", "About", None, QtGui.QApplication.UnicodeUTF8))
        
        #Generem els connectors. Son crides a funcions quan pasa alguna cosa.
        self.connect(self.tabWidget,QtCore.SIGNAL('currentChanged(int)'), self.manageKwicSort)
        self.connect(self.tabWidget,QtCore.SIGNAL('currentChanged(int)'), self.manageKwicSort)
        self.connect(self.plotZoom,QtCore.SIGNAL('valueChanged(int)'), self.managePlotZoom)
        self.connect(self.sortL1,QtCore.SIGNAL('valueChanged(int)'), self.manageSort1)
        self.connect(self.sortL2,QtCore.SIGNAL('valueChanged(int)'), self.manageSort2)
        self.connect(self.sortL3,QtCore.SIGNAL('valueChanged(int)'), self.manageSort3)
        self.connect(self.actionOpen_File, QtCore.SIGNAL('triggered()'), self.openFile)
        self.connect(self.actionOpen_Dir, QtCore.SIGNAL('triggered()'), self.openDir)
        self.connect(self.actionClose_File, QtCore.SIGNAL('triggered()'), self.closeFile)
        self.connect(self.actionClose_All_Files, QtCore.SIGNAL('triggered()'), self.closeAllFiles)
        self.connect(self.buttonAbout, QtCore.SIGNAL('triggered()'), self.about)
        self.connect(self.buttonGlobalSettings, QtCore.SIGNAL('triggered()'), self.globalSettings)
        self.connect(self.buttonToolPreferences, QtCore.SIGNAL('triggered()'), self.toolPreferences)
        self.connect(self.advancedButton, QtCore.SIGNAL('clicked()'), self.advancedsearch)
        self.connect(self.concKwikList, QtCore.SIGNAL('clicked(const QModelIndex&)'), self.concKwikListclicked)
        self.connect(self.WLwordList, QtCore.SIGNAL('clicked(const QModelIndex&)'), self.WLwordListclicked)
        self.connect(self.startButton, QtCore.SIGNAL('clicked()'), self.start)
        self.connect(self.sortButton, QtCore.SIGNAL('clicked()'), self.sort)
        self.connect(self.stopButton, QtCore.SIGNAL('clicked()'), self.stop)
        self.connect(self.searchTerm, QtCore.SIGNAL('currentIndexChanged(int)'), self.searchTermSelect)
        self.connect(self.advancedsearchwin.ventana.searchTerm, QtCore.SIGNAL('currentIndexChanged(int)'), self.searchTermSelect)
        self.connect(self.searchTerm, QtCore.SIGNAL('editTextChanged(const QString&)'), self.searchTermEdit)
        self.connect(self.advancedsearchwin.ventana.searchTerm, QtCore.SIGNAL('editTextChanged(const QString&)'), self.searchTermEdit)
        self.connect(self.words_checkbox, QtCore.SIGNAL('stateChanged(int)'), self.wordCheck)
        self.connect(self.advancedsearchwin.ventana.words_checkbox, QtCore.SIGNAL('stateChanged(int)'), self.wordCheck)
        self.connect(self.case_checkbox, QtCore.SIGNAL('stateChanged(int)'), self.caseCheck)
        self.connect(self.advancedsearchwin.ventana.case_checkbox, QtCore.SIGNAL('stateChanged(int)'), self.caseCheck)
        self.connect(self.regex_checkbox, QtCore.SIGNAL('stateChanged(int)'), self.regexCheck)
        self.connect(self.advancedsearchwin.ventana.regex_checkbox, QtCore.SIGNAL('stateChanged(int)'), self.regexCheck)
        self.connect(self.fileList, QtCore.SIGNAL('clicked(const QModelIndex&)'), self.fileClicked)
        self.connect(self.hitLocation, QtCore.SIGNAL('valueChanged(int)'), self.fileSetCursor)
        
        #Apuntem al quadre Total No. (nombre d'arxius) quan el nombre d'arxius canvia.
        self.filesModel.numberPopulated.connect(self.setFileNum)

        #Amaguem diverses parts per defecte (amb el primer tab obert).
        #Aquestes parts es mostren o amaguen depenent del tab que tinguem obert.
        self.PlotZoomFrame.hide()
        self.HitLocationFrame.hide()
        self.WindowSpanFrame.hide()
        self.WLSortByFrame.hide()
        self.SortByFrame.hide()
        self.DisplayOptionsFrame.hide()
        self.HitLocationBFrame.hide()
        
        #Definim el cursor del quadre de text a FileView. Ens servira per marcar i seleccionar text. 
        self.cursor = QTextCursor(self.fView.document())
        
        #Desactivo els tabs no implementats.
        """Eliminar aquestes dos linies per activar els tabs."""
        self.tabWidget.setTabEnabled(1,False)
        self.tabWidget.setTabEnabled(3,False)
       

    #CONTROL DEL TABPANEL
    
    #Amaga o mostra diferentes parts depenent del tab que es seleccioni.
    def manageKwicSort(self):
        if self.tabWidget.currentIndex() == 0:
            self.KwicSortFrame.show()
            self.PlotZoomFrame.hide()
            self.sortButton.show()
            self.HitLocationFrame.hide()
            self.WindowSpanFrame.hide()
            self.WLSortByFrame.hide()
            self.SortByFrame.hide()
            self.ConcHitsFrame.show()
            self.SWinSizeFrame.show()
            self.DisplayOptionsFrame.hide()
            self.HitLocationBFrame.hide()
        elif self.tabWidget.currentIndex() == 1:
            self.KwicSortFrame.hide()
            self.PlotZoomFrame.show()
            self.sortButton.hide()
            self.HitLocationFrame.hide()
            self.WindowSpanFrame.hide()
            self.WLSortByFrame.hide()
            self.SortByFrame.hide()
            self.ConcHitsFrame.show()
            self.SWinSizeFrame.hide()
            self.DisplayOptionsFrame.hide()
            self.HitLocationBFrame.hide()
        elif self.tabWidget.currentIndex() == 2:
            self.KwicSortFrame.hide()
            self.PlotZoomFrame.hide()
            self.sortButton.hide()
            self.WindowSpanFrame.hide()
            self.WLSortByFrame.hide()
            self.SortByFrame.hide()
            self.HitLocationFrame.show()
            self.ConcHitsFrame.hide()
            self.SWinSizeFrame.hide()
            self.DisplayOptionsFrame.hide()
            self.HitLocationBFrame.hide()
        elif self.tabWidget.currentIndex() == 3:
            self.KwicSortFrame.hide()
            self.PlotZoomFrame.hide()
            self.sortButton.show()
            self.HitLocationFrame.hide()
            self.WindowSpanFrame.show()
            self.WLSortByFrame.hide()
            self.SortByFrame.show()
            self.ConcHitsFrame.hide()
            self.SWinSizeFrame.hide()
            self.DisplayOptionsFrame.hide()
            self.HitLocationBFrame.hide()
        elif self.tabWidget.currentIndex() == 4:
            self.KwicSortFrame.hide()
            self.PlotZoomFrame.hide()
            self.sortButton.show()
            self.HitLocationFrame.hide()
            self.WindowSpanFrame.hide()
            self.SortByFrame.hide()
            self.WLSortByFrame.show()
            self.ConcHitsFrame.hide()
            self.SWinSizeFrame.hide()
            self.DisplayOptionsFrame.show()
            self.HitLocationBFrame.show()
        else:
            self.KwicSortFrame.hide()
            self.PlotZoomFrame.hide()
            self.sortButton.hide()
            self.HitLocationFrame.hide()
            self.WindowSpanFrame.hide()
            self.WLSortByFrame.hide()
            self.WLSortByFrame.hide()
            self.ConcHitsFrame.hide()
            self.SWinSizeFrame.hide()
            self.DisplayOptionsFrame.hide()
            self.HitLocationBFrame.hide()
    #FI DE manageKwicSort
    
    
    
        
    #CONTROL (customització) DELS SPINBOX
    
    #Controla el comportament del SpinBox de Zoom al tab Concordance Plot
    #Aquest tab no te un salt definit, ja que es potencial (funciona amb potencies de 2)
    def managePlotZoom(self):
        if((self.plotZoom.value()-1)>=(2**self.zoom)):
            self.zoom = self.zoom + 1
        elif((self.plotZoom.value()+1)<=(2**self.zoom)):
            self.zoom = self.zoom - 1
        self.plotZoom.setValue(2**self.zoom)

    #Controla el comportament del SpinBox Level 1 a Concordance.
    #Aquest no mostra mai nombres negatius, i mostra L o R.
    def manageSort1(self, i):
        if(abs(self.sort1)!=abs(i)):
            suffix = self.sortL1.suffix()
            if(suffix=="L"):
                if(i>abs(self.sort1)):
                    self.sort1 = self.sort1 + 1
                else:
                    self.sort1 = self.sort1 - 1
            else:
                if(i>abs(self.sort1)):
                    self.sort1 = self.sort1 + 1
                else:
                    self.sort1 = self.sort1 - 1
            if(self.sort1==1):
                self.sortL1.setSuffix("R")
            if(self.sort1==0):
                self.sortL1.setSuffix("")
                self.sortL1.setValue(0)
            if(self.sort1==-1):
                self.sortL1.setSuffix("L")
            if(self.sort1<0):
                a = self.sort1
                self.sortL1.setValue(abs(a))

    #Controla el comportament del SpinBox Level 2 a Concordance.
    #Aquest no mostra mai nombres negatius, i mostra L o R.
    def manageSort2(self, i):
        if(abs(self.sort2)!=abs(i)):
            suffix = self.sortL2.suffix()
            if(suffix=="L"):
                if(i>abs(self.sort2)):
                    self.sort2 = self.sort2 + 1
                else:
                    self.sort2 = self.sort2 - 1
            else:
                if(i>abs(self.sort2)):
                    self.sort2 = self.sort2 + 1
                else:
                    self.sort2 = self.sort2 - 1
            if(self.sort2==1):
                self.sortL2.setSuffix("R")
            if(self.sort2==0):
                self.sortL2.setSuffix("")
                self.sortL2.setValue(0)
            if(self.sort2==-1):
                self.sortL2.setSuffix("L")
            if(self.sort2<0):
                a = self.sort2
                self.sortL2.setValue(abs(a))

    #Controla el comportament del SpinBox Level 3 a Concordance.
    #Aquest no mostra mai nombres negatius, i mostra L o R.
    def manageSort3(self, i):
        if(abs(self.sort3)!=abs(i)):
            suffix = self.sortL3.suffix()
            if(suffix=="L"):
                if(i>abs(self.sort3)):
                    self.sort3 = self.sort3 + 1
                else:
                    self.sort3 = self.sort3 - 1
            else:
                if(i>abs(self.sort3)):
                    self.sort3 = self.sort3 + 1
                else:
                    self.sort3 = self.sort3 - 1
            if(self.sort3==1):
                self.sortL3.setSuffix("R")
            if(self.sort3==0):
                self.sortL3.setSuffix("")
                self.sortL3.setValue(0)
            if(self.sort3==-1):
                self.sortL3.setSuffix("L")
            if(self.sort3<0):
                a = self.sort3
                self.sortL3.setValue(abs(a))
  
  
  
    #CONTROL DE FITXERS
    
    #Obre un o mes arxius. Crida a la finestra Open File.
    def openFile(self):
        filename = QtGui.QFileDialog.getOpenFileNames(self, 'Open file','','Text Files (*.txt);;HTML Files (*.html *.htm);;XML Files (*.xml);;AntConc Files (*.ant);;All Files (*.*)')
        for i in range(len(filename)):
            self.filesModel.addFile(filename[i])

    #Obre els arxius en undirectori. Crida a la finestra Open Directory.
    #Funciona recursivament, és a dir que obre, també, els arxius de tots els directoris fills.
    def openDir(self):
        dirname = QtGui.QFileDialog.getExistingDirectory(self, "Open Directory","", QtGui.QFileDialog.ShowDirsOnly | QtGui.QFileDialog.DontResolveSymlinks)
        if dirname != "":
            self.filesModel.setDirPath(dirname)
        
    #Tanca els arxius seleccionats.
    def closeFile(self):
        indexes = []
        for i in range(len(self.fileList.selectedIndexes())):
            indexes.append(self.fileList.selectedIndexes()[i].row())
        for i in sorted(indexes,key=int,reverse=True):
            self.filesModel.removeFile(i)
        
    #Tanca tots els arxius.
    def closeAllFiles(self):
        self.filesModel.removeAllFiles()
        
    #Posa el valor num a Total No. (el nombre total de fitxers)
    def setFileNum(self,num):
        self.totalFiles.setText(str(num))

    #Actualitza File View en clicar un fitxer.
    def fileClicked(self,i):
        #Obté el contingut de l'arxiu
        self.fView.document().setPlainText(files.get(i.row()))
        #Obté les cerques actuals (s'han de marcar en blau)
        if searchWords.useSearchList:
            text = self.advancedsearchwin.searchList.toPlainText().split("\n")
        else:
            text = self.searchTerm.currentText()

        if searchWords.useContextWords:
            context = searchWords.contextWords
        else:
            context = ""
        
        self.fileResults = func.searchTerms(text,self.fView.document().toPlainText(),0,-1,self.case_checkbox.isChecked(),self.words_checkbox.isChecked(),self.regex_checkbox.isChecked(),context,searchWords.contextFrom,searchWords.contextTo)
        fileText = self.fView.document().toPlainText()
        start = 0 
        self.ex = []
        #Introdueix la informació al quadre de File View
        self.fViewFile.setText(i.data())
        
        if self.fileResults is not False:
            #self.fView.document().setPlainText(fileText)
            for x in range(len(self.fileResults)):
                #Marca en blau TOTES les coincidencies
                #Posa el cursor al lloc
                self.cursor.setPosition(self.fileResults[x][0],mode= QTextCursor.MoveAnchor)
                self.cursor.setPosition((self.fileResults[x][0]+len(self.fileResults[x][1])),mode= QTextCursor.KeepAnchor)
                #Crea un nou element a ex per cada coincidencia
                self.ex.append(QTextEdit.ExtraSelection())
                #Dona les propietats i el color.
                self.ex[x].cursor = self.cursor
                self.ex[x].format.setProperty(QTextFormat.FullWidthSelection, True)
                self.ex[x].format.setBackground(QColor("white"))
                self.ex[x].format.setForeground(QColor("blue"))
               
            #Omple la variable amb el nombre de coincidencies
            self.fViewHits.setText(str(len(self.fileResults)))                
            #Posa el Hit Location a 1 i limita amb el nombre de coincidencies.
            self.hitLocation.setMaximum(len(self.fileResults))
            self.hitLocation.setValue(1)
            
        else:
            #Omple la variable amb el nombre de coincidencies
            #Posa el Hit Location a 1 i limita amb el nombre de coincidencies.
            self.fViewHits.setText("0")        
            self.hitLocation.setMaximum(1)
         
        #Assigna les seleccions en blau al quadre de text per que apareguin.
        self.fView.setExtraSelections(self.ex)
        
        #Marca (selecciona) la primera coincidencia.
        if self.fileResults is not False:
            self.cursor.setPosition(self.fileResults[0][0],mode= QTextCursor.MoveAnchor)
            self.cursor.setPosition((self.fileResults[0][0]+len(self.fileResults[0][1])),mode= QTextCursor.KeepAnchor)
            self.fView.setTextCursor(self.cursor)
        
        #Dona el focus a la finestra
        self.fView.setFocus()

    #Marca (selecciona) la coincidencia en posició i.
    def fileSetCursor(self,i):
        if self.fileResults is not False:
            self.cursor.setPosition(self.fileResults[i-1][0],mode= QTextCursor.MoveAnchor)
            self.cursor.setPosition((self.fileResults[i-1][0]+len(self.fileResults[i-1][1])),mode= QTextCursor.KeepAnchor)
            self.fView.setTextCursor(self.cursor)        
        self.fView.setFocus()
    
    
    
    
    #CONTROL DE FINESTRES
    
    #Obre la finestra About
    def about(self):
        self.aboutwin.show()
    
    #Obre la finestra Global Settings
    def globalSettings(self):
        QMessageBox.information (self, "Alert", "This window is not usable.\nThe functionality of this window is not working. Any change you do here is not saved or doesn't affect the tool behavior.", buttons = QMessageBox.Ok, defaultButton = QMessageBox.Ok)
        self.globalsettingswin.show()
    
    #Obre la finestra Tool Preferences
    def toolPreferences(self):
        QMessageBox.information (self, "Alert", "This window is not usable.\nThe functionality of this window is not working. Any change you do here is not saved or doesn't affect the tool behavior.", buttons = QMessageBox.Ok, defaultButton = QMessageBox.Ok)
        self.toolpreferenceswin.show()

    #Obre la finestra Advanced Search
    def advancedsearch(self):
        #Carreguem la informació que està guardada a la classe searchWords (al fitxer pfc_func.py)
        self.advancedsearchwin.ventana.termListCheckbox.setChecked(searchWords.useSearchList)
        self.advancedsearchwin.ventana.contextWordCheckbox.setChecked(searchWords.useContextWords)
        
        #Llistat de paraules
        text = ""
        for word in searchWords.searchList:
            text = text + "\n" + word
        self.advancedsearchwin.searchList.setPlainText(text[1:])
        
        #Llistat de Context Words 
        self.advancedsearchwin.ventana.contextWordList.clear()
        for word in searchWords.contextWords:
            self.advancedsearchwin.ventana.contextWordList.addItem(word)
        
        #ContextFrom
        self.advancedsearchwin.ventana.contextFrom.setValue(abs(searchWords.contextFrom))
        if searchWords.contextFrom == 0:
            self.advancedsearchwin.ventana.contextFrom.setSuffix("")
        elif searchWords.contextFrom < 0:
            self.advancedsearchwin.ventana.contextFrom.setSuffix("L")
        elif searchWords.contextFrom > 0:
            self.advancedsearchwin.ventana.contextFrom.setSuffix("R")
        
        #ContextTo
        self.advancedsearchwin.ventana.contextTo.setValue(abs(searchWords.contextTo))
        if searchWords.contextTo == 0:
            self.advancedsearchwin.ventana.contextTo.setSuffix("")
        elif searchWords.contextTo < 0:
            self.advancedsearchwin.ventana.contextTo.setSuffix("L")
        elif searchWords.contextTo > 0:
            self.advancedsearchwin.ventana.contextTo.setSuffix("R")

        #Mostra la finestra
        self.advancedsearchwin.show()



    #ACCIONS EN CLICAR LLISTATS
    
    #Quan una linia a la llista KWIK del tab Concordance es clicada obre el tab File View
    #En el tab File View resalta la coincidencia.
    def concKwikListclicked(self,num):
        fileIndex = self.fileList.model().findByName(kwikobj.fileList[num.row()])
        index = self.fileList.model().createIndex(fileIndex,0)
        self.fileClicked(index)
        self.tabWidget.setCurrentIndex(2)
        pos = self.results[num.row()][5]
        self.hitLocation.setValue(pos)
    
    def WLwordListclicked(self,num):
        word = num.data()
        
        #Afegeix la paraula i la selecciona.
        self.searchTerm.insertItem(0,word,userData=None)
        self.searchTerm.setCurrentIndex(0)
        
        #Desactiva el checkbox "Use search term(s) from list below" a la finestra Advanced Search
        searchWords.useSearchList = False
        self.tabWidget.setCurrentIndex(0)
        self.start()
            
           
               
    
    #ACCIONS DELS BOTONS
    
    #Start (per totes les pestanyes)
    def start(self):
        
        #Omple fileView si nomes hi ha un arxiu seleccionat
        if len(self.fileList.selectedIndexes()) == 1:
            self.fileClicked(self.fileList.selectedIndexes()[0])
        
        #Posem la barra de progres a zero
        self.progressBar.setValue(0)
        
        #Afegeix la nova cerca al combobox i la selecciona.
        self.searchTerm.insertItem(0,self.searchTerm.currentText(),userData=None)
        self.searchTerm.setCurrentIndex(0)
        
        if searchWords.useSearchList:
            text = self.advancedsearchwin.searchList.toPlainText().split("\n")
        else:
            text = self.searchTerm.currentText()

        if searchWords.useContextWords:
            context = searchWords.contextWords
        else:
            context = ""
        
        #Si el tab Concordance esta activat fes la cerca per aquest
        if self.tabWidget.currentIndex() == 0:
            #Neteja (buida) les variables
            kwikobj.clean(None)
            self.results = []
            for i in range(len(files.content)):
                #Incrementem la barra de progres per cada arxiu
                progressVal = (100*i)/len(files.content)
                self.progressBar.setValue(progressVal)
                #Fem la cerca per cada fitxer
                results = func.searchTerms(text,files.content[i],0,-1,self.case_checkbox.isChecked(),self.words_checkbox.isChecked(),self.regex_checkbox.isChecked(),context,searchWords.contextFrom,searchWords.contextTo)
                if results != False:
                    order = 0
                    for result in results:
                        #Obtenim la linia de cada coincidencia
                        l = func.returnLine(result,files.content[i],self.searchWindowSize.value())
                        #Afegim codi HTML a la linia per colorejar la expressió (la coincidencia)
                        paintedLine = func.paintLine(l[0],l[1],result[1],[-1,-1],[-1,-1],[-1,-1])
                        #Afegim la paraula a kwikobj
                        kwikobj.kwikList.append(paintedLine)
                        kwikobj.fileList.append(self.fileList.model().fileList[i])
                        #Afegim a result el nom del fitxer, la linia i la posició de l'expressió a la linia
                        result.append(self.fileList.model().fileList[i])
                        result.append(l[0])
                        result.append(l[1])
                        #Afegim a result el numero de coincidencia en el fitxer (és un incremental)
                        order = order + 1
                        result.append(order)
                        #Indiquem el numero de coincidencies trobades fins ara (Es veurà com va incrementant-se).
                        self.concordanceHits.setText(str(len(kwikobj.kwikList)))
                    
                    #Afegeix els resultats a la llista results de MainWindow
                    self.results.extend(results)
                    
            #Reinicia les QListView per que apareixi la nova informació.        
            self.concFileList.model().reset()
            self.concKwikList.model().reset()
            self.concHitList.model().reset()
            #print(self.results)
    
            self.progressBar.setValue(100)
            #Indiquem el numero de coincidencies final. No es realment necessari, però servirà per assegurar-nos si toquem codi més endavant.
            self.concordanceHits.setText(str(len(kwikobj.kwikList)))
        
        #Si el tab WordList esta activat, la cerca és la següent
        if self.tabWidget.currentIndex() == 4:
            self.progressBar.setValue(0)
            
            #Neteja (buida) wlobj
            wlobj.clean(None)
            
            #Obtenim els valors sortD, sortF i case.
            """
            sortD: Direcció ascendent o descendent de l'ordenació. Depen de si "Invert Order" està checked.
            sortF: La opció (Field, tot i que realment no és el camp) d'ordenació. Extret del llistat "Sort by" 
            case: Distingir o no entre majúscules i minúscules. Depén del checkbox "Treat all data as lowercase"
            """
            sortD = self.WLInvertOrder.isChecked()
            if sortD:
                sortD = 'desc'
            sortF = self.WLSortBy.currentText()[8:].replace(" ","")
            case = self.WLAllLowerCase.isChecked()
           
            results = func.wordList(sortF, sortD, case)
            
            #Si hi ha resultats omple wlobj
            words = 0
            if results != None:
                for result in results:
                    wlobj.addItem(None,result[2],result[0],"")
                    words = words + result[2]
            
            #Omple les variables superiors
            self.wListHits = "0"
            self.wListTotalTypes.setText(str(len(results)))
            self.wListTotalTokens.setText(str(words))
            
            self.WLwordList.model().reset()
            self.WLfreqList.model().reset()
            self.WLrankList.model().reset()
            self.WLlemmaList.model().reset()
                
            self.progressBar.setValue(100)
    #FI DE start
            
    #Sort Concordance
    def sort(self):
        
        #Obté els resultats
        wordList = self.results
        #Obté els valors per al sorting amb els 3 nivells d'ordenació
        l1=None
        l2=None
        l3=None
        if self.sort1_checkbox.isChecked() is True:
            l1 = str(self.sortL1.value())+str(self.sortL1.suffix())
        if self.sort2_checkbox.isChecked() is True:
            l2 = str(self.sortL2.value())+str(self.sortL2.suffix())
        if self.sort3_checkbox.isChecked() is True:
            l3 = str(self.sortL3.value())+str(self.sortL3.suffix())
            
        #Rep l'ordre
        order = func.sortWList(wordList,l1,l2,l3)
        
        #Creem una copia dels objectes localment
        kwikList = kwikobj.kwikList
        fileList = kwikobj.fileList
        results = self.results
        
        #Buidem els objectes i resetejem els models per que mostrin el canvi.
        kwikobj.kwikList = []
        kwikobj.fileList = [] 
        self.results = []  
        
        self.concFileList.model().reset()
        self.concKwikList.model().reset()    
        
        #Introduïm la informació des de la copia de manera ordenada.
        #Afegim també informació a results amb la posició de les paraules seleccionades per a l'ordenació.
        for i in range(len(kwikList)):
            if self.sort1_checkbox.isChecked() is True:
                pos1 = func.wordPos(results[order[i]][4], results[order[i]][3], l1, l1)
            else:
                pos1 = [-1,-1]
            if self.sort2_checkbox.isChecked() is True:
                pos2 = func.wordPos(results[order[i]][4], results[order[i]][3], l2, l2)
            else:
                pos2 = [-1,-1]
            if self.sort3_checkbox.isChecked() is True:
                pos3 = func.wordPos(results[order[i]][4], results[order[i]][3], l3, l3)
            else:
                pos3 = [-1,-1]
            if(len(results[order[i]]))>6:
                results[order[i]][6] = pos1
                results[order[i]][7] = pos2
                results[order[i]][8] = pos3
            else:
                results[order[i]].append(pos1)
                results[order[i]].append(pos2)
                results[order[i]].append(pos3)
            
            #Afegim la linia pintada (amb HTML) a la llista KWIK
            kwikobj.kwikList.append(func.paintLine(results[order[i]][3],results[order[i]][4],results[order[i]][1],results[order[i]][6],results[order[i]][7],results[order[i]][8]))
            #Afegim el nom del fitxer
            kwikobj.fileList.append(results[order[i]][2])
            #Afegim el resultat al llistat de results de MainWindow
            self.results.append(results[order[i]])
        #Resetejem les QListView per mostrar els canvis.
        self.concFileList.model().reset()
        self.concKwikList.model().reset()
    #FI DE sort
        
    #Funció STOP. Només funcionarà si hi ha THREADING
    def stop(self):
        print("stop")  
        #No funciona, l'eina no funciona actualment amb threads.
    
     
    
    
    
    #CONTROL DE ADVANCED WINDOW 
            
    #Quan es selecciona un element al combo Search Term actualitza tant en Advanced com en MainWindow.
    def searchTermSelect(self,i):
        self.searchTerm.setCurrentIndex(i)
        self.advancedsearchwin.ventana.searchTerm.setCurrentIndex(i)

    #Quan s'edita un element al combo Search Term actualitza tant en Advanced com en MainWindow.
    def searchTermEdit(self,text):
        self.advancedsearchwin.ventana.searchTerm.lineEdit().setText(text)
        self.searchTerm.lineEdit().setText(text)
        
    #Quan es canvia el checkbox Word actualitza tant en Advanced com en MainWindow.
    def wordCheck(self,i):
        if(i == 2):
            self.advancedsearchwin.ventana.words_checkbox.setChecked(True)
            self.words_checkbox.setChecked(True)
        elif(i==0):
            self.advancedsearchwin.ventana.words_checkbox.setChecked(False)
            self.words_checkbox.setChecked(False)

    #Quan es canvia el checkbox Case actualitza tant en Advanced com en MainWindow.
    def caseCheck(self,i):
        if(i == 2):
            self.advancedsearchwin.ventana.case_checkbox.setChecked(True)
            self.case_checkbox.setChecked(True)
        elif(i==0):
            self.advancedsearchwin.ventana.case_checkbox.setChecked(False)
            self.case_checkbox.setChecked(False)

    #Quan es canvia el checkbox RegEx actualitza tant en Advanced com en MainWindow.
    def regexCheck(self,i):
        if(i == 2):
            self.advancedsearchwin.ventana.regex_checkbox.setChecked(True)
            self.regex_checkbox.setChecked(True)
            self.case_checkbox.setEnabled(False)
            self.words_checkbox.setEnabled(False)
            self.advancedsearchwin.ventana.case_checkbox.setEnabled(False)
            self.advancedsearchwin.ventana.words_checkbox.setEnabled(False)
        elif(i==0):
            self.advancedsearchwin.ventana.regex_checkbox.setChecked(False)
            self.regex_checkbox.setChecked(False)
            self.case_checkbox.setEnabled(True)
            self.words_checkbox.setEnabled(True)
            self.advancedsearchwin.ventana.case_checkbox.setEnabled(True)
            self.advancedsearchwin.ventana.words_checkbox.setEnabled(True)

    
    
    
    
    #CONTROL D'EVENTS
    
    #En tancar MainWindow mostra un missatge demanant si estem segurs.
    def closeEvent(self, event):
        answer = QMessageBox.question (self, "Message", "Are you sure you want to exit?", buttons = QMessageBox.Yes | QMessageBox.No, defaultButton = QMessageBox.No)
        if answer == QMessageBox.No:
            event.ignore()
